# -*- coding: ISO-8859-1 -*-

##############################################################################
#
# Copyright (c) 2006 Alexis Roda. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
##############################################################################

"""

$Id: ziptool.py 17 2006-05-21 22:45:01Z alex $
"""

__version__ = "$Revision: $"
# $Source: $

import mimetypes
import os.path

from AccessControl import ClassSecurityInfo
from Acquisition import aq_base
from Globals import InitializeClass
from Globals import DTMLFile
#from OFS.Folder import manage_addFolder
from OFS.Image import manage_addImage
from OFS.Image import manage_addFile
from OFS.SimpleItem import SimpleItem

import permissions
from zipfolder import manage_addZipFolder
import archiverregistry
import compressorregistry
from exceptions import BadArchiveError
from exceptions import UnsupportedFormatError


class ZipTool(SimpleItem) :
    """
    """

    security = ClassSecurityInfo()

    id = 'ziptool'
    meta_type = 'ziptool:ziptool'

    manage_options = ({'label'  : 'Overview',
                       'action' : 'manage_overview'},
                      ) + SimpleItem.manage_options

    manage_overview = DTMLFile('dtml/manage_overview', globals())

    security.declareProtected(permissions.use_zip_services, 'unzip')
    def unzip(self, archname, file_, destination_path) :
        """
        """
        file_ = compressorregistry.decompress(file_)
        handler = archiverregistry.getHandler(file_)
        if handler is None :
            raise UnsupportedFormatError()

        try:
            archive = handler(archname, file_)
        except BadArchiveError:
            raise

        dest_container = self.restrictedTraverse(destination_path)
        for member in archive :
            dirname, filename = os.path.split(member.filename())
            folder = self._create_folder(dest_container, dirname)
            self._create_member(folder, filename, member)
        archive.close()

    security.declarePrivate('_create_folder')
    def _create_folder(self, container, name) :
        for part in name.split('/') :
            if not hasattr(aq_base(container), part) :
                manage_addZipFolder(container, part)
            container = getattr(container, part)
        return container

    security.declarePrivate('_create_member')
    def _create_member(self, container, name, member) :
        mimetype, encoding = mimetypes.guess_type(name)
        if mimetype is None :
            mimetype = 'application/octet-stream'
        type_, subtype = mimetype.split('/')
        if type_ == 'image' :
            manage_addImage(container, name,
                            file=member.read(),
                            content_type=mimetype)
        else:
            manage_addFile(container, name,
                           file=member.read(),
                           content_type=mimetype)

InitializeClass(ZipTool)


def manage_addZipTool(self, REQUEST=None) :
    """'
    """
    zt = ZipTool()
    self._setObject(zt.id, zt)
    if REQUEST is not None :
        REQUEST.RESPONSE.redirect('./manage_main')
